# AsteriskLint -- an Asterisk PBX config syntax checker
# Copyright (C) 2015-2016  Walter Doekes, OSSO B.V.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from ..base import AppBase


class Pickup(AppBase):
    pass


class PickupChan(AppBase):
    pass


class PickupOld1v4(AppBase):
    """
    This is not in any main Asterisk branch. Quickest fix was to add
    here for now.

    NOTE: https://issues.asterisk.org/jira/browse/ASTERISK-26464
    """
    pass


def register(app_loader):
    for app in (
            Pickup,
            PickupChan,
            PickupOld1v4):
        app_loader.register(app())
