# AsteriskLint -- an Asterisk PBX config syntax checker
# Copyright (C) 2015-2016  Walter Doekes, OSSO B.V.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from ..base import FuncBase


class QUEUE_EXISTS(FuncBase):
    pass


class QUEUE_MEMBER(FuncBase):
    pass


class QUEUE_MEMBER_COUNT(FuncBase):
    pass


class QUEUE_MEMBER_LIST(FuncBase):
    pass


class QUEUE_MEMBER_PENALTY(FuncBase):
    pass


class QUEUE_VARIABLES(FuncBase):
    pass


class QUEUE_WAITING_COUNT(FuncBase):
    pass


def register(func_loader):
    for func in (
            QUEUE_EXISTS, QUEUE_MEMBER, QUEUE_MEMBER_COUNT,
            QUEUE_MEMBER_LIST, QUEUE_MEMBER_PENALTY,
            QUEUE_VARIABLES, QUEUE_WAITING_COUNT):
        func_loader.register(func())
